/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				blendCollage.c
 *	To run, type:
 *	blendCollage <file.rgb> [<file2.rgb>] ... 
 *
 *	Try "blendCollage /usr/demos/data/textures/*.rgb"
 *
 *	Up to MAX_IMAGES can be read.  blendCollage will randomly
 *	display a new image each time the left mouse button is 
 *	pressed; different blending functions can be enable to 
 *	create different effects.
 *		Escape key		- exit the program
 *		<b>			- change blend function
 *		LEFT Mousebutton, down  - randomly choose a new image 
 *
 *	David Marsland, MTS, SGI Education R & D, 1993
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <aux.h>
#include <math.h>
#include <stdio.h>
#include "rgbImageFile.h"

#define MAX_IMAGES 100

/*  Function Prototypes  */

GLvoid  initialize( int argc, char *argv[] );
GLvoid  drawScene( GLvoid );
GLvoid  printHelp( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid	newImage( AUX_EVENTREC * );
GLvoid  blendFuncCycle( GLvoid );

/* Global Variables */
unsigned int *images[MAX_IMAGES] = { NULL };
int imageWidths[MAX_IMAGES], imageHeights[MAX_IMAGES];
GLsizei     screenWidth, screenHeight;
GLsizei     windowWidth, windowHeight;
GLint       xPos, yPos;
GLint       imageCount, currentImage = 0;

main ( int argc, char *argv[])
{
	if (argc < 2) {
		fprintf (stderr, 
		  "usage: blendCollage <imageFile1> [<imageFile2> ... ]\n");
		exit (1);
	}
	
	initialize( argc, argv );

	auxMainLoop( drawScene );

}
		
GLvoid
resize( GLsizei width, GLsizei height )
{
	GLint   matrixMode;

	windowWidth = width;
	windowHeight = height;

	glViewport( 0, 0, windowWidth, windowHeight);

	glGetIntegerv( GL_MATRIX_MODE, &matrixMode );
	glLoadIdentity();
	glOrtho( -0.5, (GLfloat) windowWidth-0.5,
		-0.5, (GLfloat) windowHeight-0.5, -1.0, 1.0 );
	glMatrixMode( matrixMode );
	glLoadIdentity();
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, "\n\
blendCollage create an alpha blended collage of images\n\n\
Escape key		- exit the program\n\
<b> key			- change blend function\n\
LEFT Mousebutton, down 	- randomly choose a new image \n\n");
}

GLvoid  
blendFuncCycle( GLvoid )
{
	static int whichBlendFunc = 0;
	
	whichBlendFunc = (whichBlendFunc + 1) % 3;
	
	switch (whichBlendFunc)
	{
	case 0:
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		fprintf(stdout, "\
		    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA )\n" );
		break;
	case 1:
		glBlendFunc( GL_SRC_ALPHA, GL_ONE );
		fprintf(stdout, "\
		    glBlendFunc( GL_SRC_ALPHA, GL_ONE )\n" );
		break;
	case 2:
		glBlendFunc( GL_ONE, GL_ZERO );
		fprintf(stdout, "\
		    glBlendFunc( GL_ONE, GL_ZERO )\n" );
		break;
	default:
		break;
	}
}


GLvoid
initialize( int argc, char *argv[] )
{
	int i;
	
	imageCount = argc - 1;
	if (imageCount > MAX_IMAGES)
		imageCount = MAX_IMAGES;

	srand48( imageCount );

	/* read in all image files specifed as command line args */
	for ( i = 0; i < imageCount; i++ )
	{
		images[i] = rgbReadImageFileAlpha( argv[i+1], 
			&imageWidths[i], &imageHeights[i], 0.5);
	}
	
	auxGetScreenSize( &screenWidth, &screenHeight );
	auxInitPosition( screenWidth/8, screenHeight/8, 
		3*screenWidth/4, 3*screenHeight/4 );
	auxInitDisplayMode( AUX_RGBA | AUX_SINGLE );
	
	auxInitWindow( "blendCollage.c      `b' key - change blend function   +  LEFTMOUSE" );
	auxGetSize( &windowWidth,  &windowHeight );

	auxAnimation( AUX_ON );
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClear( GL_COLOR_BUFFER_BIT );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, newImage );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_b, blendFuncCycle );

	auxReshapeFunc( resize );

	glViewport( 0, 0, windowWidth, windowHeight);

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( -0.5, (GLfloat) windowWidth-0.5,
		-0.5, (GLfloat) windowHeight-0.5, -1.0, 1.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	printHelp();

	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
}

GLvoid
newImage( AUX_EVENTREC *event )
{
	/* grab the new mouse location */
	xPos = event->data[AUX_MOUSEX];
	yPos = event->data[AUX_MOUSEY];

	yPos = windowHeight - yPos;

	/* pick a new image index */
	currentImage = (int) (imageCount * drand48());
}

GLvoid
drawScene( GLvoid )
{
	xPos += (2.0*drand48() - 1.0)*.05*screenWidth;
	yPos += (2.0*drand48() - 1.0)*.05*screenHeight;
	xPos = fabs( xPos );
	yPos = fabs( yPos );
	glRasterPos2f ( fmodf(xPos, windowWidth), fmodf(yPos, windowHeight) );
	glDrawPixels ( imageWidths[currentImage], imageHeights[currentImage],
			GL_RGBA, GL_UNSIGNED_BYTE, images[currentImage]);
	glFlush();
}
