/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				clip.c
 * This program demonstrates using arbitrary clipping planes.
 * SPACE key	- toggles fixed/moving clipping planes
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

/* Function Prototypes */

GLvoid initialize();
GLvoid drawScene( GLvoid );
GLvoid resize( GLsizei width, GLsizei height );
GLvoid initLighting( GLvoid );
GLvoid toggleMovingClip( GLvoid );
GLvoid printHelp( GLvoid );

GLboolean movingClipPlanes = GL_TRUE;

main ()
{
	initialize();
	auxMainLoop( drawScene );
}

/* Initialize material property, light source, lighting model, 
 * and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );
	glEnable( GL_LIGHTING );
	glEnable( GL_LIGHT0 );
	glColorMaterial( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable( GL_COLOR_MATERIAL );
}

GLvoid
initialize( )
{
	GLsizei  width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition(winSize/4, winSize/4, winSize/2, winSize/2);
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "clip   SPACEBAR - toggle fixed/moving clipping planes" );
	
	auxReshapeFunc( resize );
	auxAnimation( AUX_ON );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, toggleMovingClip );
	
	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );
	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -8.0 );
	initLighting(); 
	printHelp();
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0 ); 
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, 
"\nclip.c - This program demonstrates defining arbitrary\n"
"clipping planes.\n"
"SPACE key	- toggles fixed/moving clipping planes \n"
"Escape key	- exit the program \n\n" );
}

GLvoid
toggleMovingClip( GLvoid )
{
	movingClipPlanes = !movingClipPlanes;	
}

GLvoid
drawScene( GLvoid )
{
	GLdouble planeABCD[4] = {-1.0, 0.0, 0.0, 0.5};
	/* to reverse the clipping direction, 
	 * reverse signs of A, B, C, and D 
	 */
	GLdouble oppositePlaneABCD[4]  = {1.0, 0.0, 0.0, -0.5};

	static GLfloat angle = 0.0; 
	int i,  slices = 8;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	if ( !movingClipPlanes )
	{
		/* define clipping planes fixed in scene */
		glClipPlane (GL_CLIP_PLANE1, planeABCD);
		glClipPlane (GL_CLIP_PLANE2, oppositePlaneABCD);
	}
	
	glPushMatrix(); 
		glRotatef( angle, 0.0, 1.0, 0.0 );
		
		if ( movingClipPlanes )
		{
			/* defining clipping planes after rotation
			 * causes the clipping planes to rotate
			 */
			glClipPlane (GL_CLIP_PLANE1, planeABCD);
			glClipPlane (GL_CLIP_PLANE2, oppositePlaneABCD);
		}

		/* clip off parts of the solid torii 
		 */
		glEnable (GL_CLIP_PLANE1);

		for ( i = 0; i < slices; i++ )
		{
			glColor3f( i/10.0, i/10.0, 1.0 - i/10.0 ); 
			glPushMatrix(); 
			glRotatef( i * 360.0/slices, 0, 0, 1 );
			glTranslatef( 1.5, 0.0, 0.0 );
			glRotatef( i * 360.0/slices, 0, 1, 0 );
			auxSolidTorus( 0.25, 0.75 );
			glPopMatrix();
		}
		glDisable (GL_CLIP_PLANE1);

		/* clip off the opposite parts of the wire torii */
		glEnable (GL_CLIP_PLANE2);

		for ( i = 0; i < slices; i++ )
		{
			glColor3f( i/10.0, i/10.0, 1.0 - i/10.0 ); 
			glPushMatrix(); 
			glRotatef( i * 360.0/slices, 0, 0, 1 );
			glTranslatef( 1.5, 0.0, 0.0 );
			glRotatef( i * 360.0/slices, 0, 1, 0 );
			auxWireTorus( 0.25, 0.75 );
			glPopMatrix();
		}
		glDisable (GL_CLIP_PLANE2);
	glPopMatrix();

	auxSwapBuffers();

	angle -= 1.0;
	if (angle < -360.0)
		angle = 0.0;
}
