/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* outlineFont.c - create an outline font and display it using 
 * 		glCallLists()
 * 
 *	Escape key			- exit the program
 *	Left Mousebutton, down		- start moving the eye
 *	Left Mousebutton, up		- stop moving the eye
 *	Mouse pointer location		- position eye
 *	Middle Mousebutton, down 	- scale up
 *	Right Mousebutton, down		- scale down
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aux.h"
#include "outlineFont.h"

GLvoid  initialize( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  drawScene( GLvoid );
GLvoid  scaleUp( AUX_EVENTREC * );
GLvoid  scaleDown( AUX_EVENTREC * );
GLvoid  startMove( AUX_EVENTREC * );
GLvoid  stopMove( AUX_EVENTREC * );
GLvoid  mouseMove( AUX_EVENTREC * );
GLvoid  printHelp( GLvoid );
GLvoid  createOutlineFont( GLuint );

/* Global Variables */
GLfloat     latitude, longitude, radius;
GLboolean   moving = GL_FALSE;
GLint       xStart, yStart;

GLfloat     scaleFactor = 1.0;

static GLuint outlineBase;
char string[] = 
	"ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz";

void 
main(int argc, char **argv)
{

	initialize();

	auxMainLoop(drawScene);
}

GLvoid 
initialize(void)
{
	GLsizei    width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 8, winSize / 8, 800, 400 );
	auxInitDisplayMode( AUX_RGBA | AUX_DOUBLE );
	auxInitWindow( "outlineFont" );

	auxReshapeFunc( resize );
	auxAnimation( AUX_ON );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	/* Set up functions using the mouse */
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );
	auxMouseFunc( AUX_MOUSELOC, 0, mouseMove );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, scaleUp );
	auxMouseFunc( AUX_RIGHTBUTTON, AUX_MOUSEDOWN, scaleDown );

	glMatrixMode( GL_PROJECTION );
	glOrtho(-400.5, 399.5, -200.5, 199.5, -400.5, 399.5);
	glMatrixMode( GL_MODELVIEW );

	printHelp();

	outlineBase = glGenLists(256);
	createOutlineFont(outlineBase);

	glClearColor(0.0, 0.0, 0.0, 0.0);
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, "\noutlineFont\n\n"\
		"Escape key			- exit the program\n"\
		"Left Mousebutton, down		- start moving the eye\n"\
		"Left Mousebutton, up		- stop moving the eye\n"\
		"Mouse pointer location		- position eye\n"\
		"Middle Mousebutton, down 	- scale up\n"\
		"Right Mousebutton, down		- scale down\n\n");
}

/* Create a font which consists of the outline of each letter.
 * Put the description for each letter in a separate display list.
 */
GLvoid 
createOutlineFont(GLuint fontBase)
{
	GLint mode, i, j;

	for (i = 0; outlineFont[i][0] != END_OF_LIST; i++) {
	  glNewList(fontBase+(unsigned int)outlineFont[i][0], GL_COMPILE);
	    for (j = 1; mode = outlineFont[i][j]; j += 3) {
	      if (mode == FONT_BEGIN) {
	      	glBegin(GL_LINE_LOOP);
	      	glVertex2f((float)outlineFont[i][j+1]*FONT_SCALE,
	      	    (float)outlineFont[i][j+2]*FONT_SCALE);
	      } else if (mode == FONT_NEXT) {
	      	glVertex2f((float)outlineFont[i][j+1]*FONT_SCALE,
	      	    (float)outlineFont[i][j+2]*FONT_SCALE);
	      } else if (mode == FONT_END) {
	      	glVertex2f((float)outlineFont[i][j+1]*FONT_SCALE,
	      	    (float)outlineFont[i][j+2]*FONT_SCALE);
	      	glEnd();
	      } else if (mode == FONT_ADVANCE) {
	      	glTranslatef((float)outlineFont[i][j+1]*FONT_SCALE,
	      	    (float)outlineFont[i][j+2]*FONT_SCALE, 0.0);
	      	break;
	      }
	    }
	  glEndList();
	}
}

GLvoid 
resize( GLsizei width,  GLsizei height )
{

	glViewport(0, 0, (GLint)width, (GLint)height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-400.5, 399.5, -200.5, 199.5, -400.5, 399.5);
	glMatrixMode(GL_MODELVIEW);
}

/* Define user input functions */
GLvoid  
scaleDown( AUX_EVENTREC *event )
{
	scaleFactor -= 0.1;
	if (scaleFactor < 0.1) {
		scaleFactor = 0.1;
	}
}

GLvoid  
scaleUp( AUX_EVENTREC *event )
{
	scaleFactor += 0.1;
}

GLvoid
startMove( AUX_EVENTREC *event )
{
	moving = GL_TRUE;

	/* Record where the mouse pointer was when the user pressed 
	 * the left mouse buton 
	 */
	xStart = event->data[AUX_MOUSEX];
	yStart = event->data[AUX_MOUSEY];
}

GLvoid
stopMove( AUX_EVENTREC *event )
{
	moving = GL_FALSE;
}

GLvoid
mouseMove( AUX_EVENTREC *event )
{
	GLint     xPos, yPos;
	GLint     xOffset, yOffset;

	if ( moving ) {
		xPos = event->data[AUX_MOUSEX];
		yPos = event->data[AUX_MOUSEY];

		/* Using the stored and current mouse positions, 
		 * compute how far the mouse moved 
		 */
		xOffset = xPos - xStart;
		yOffset = yPos - yStart;

		/* Adjust the eye position based on the mouse position */
		longitude -= xOffset;
		latitude += yOffset;

		/* Update the stored mouse position for later use */
		xStart = xPos;
		yStart = yPos;
	}
}
GLvoid 
drawScene( GLvoid )
{

	glClear(GL_COLOR_BUFFER_BIT);

	glColor3f(1.0, 1.0, 1.0);

	glPushMatrix();

		glTranslatef( 0.0, 0.0, -radius);
		glRotatef( latitude, 1.0, 0.0, 0.0 );
		glRotatef( longitude, 0.0, 0.0, 1.0 );
		glScalef(scaleFactor, scaleFactor, scaleFactor);

		glPushMatrix();
			glTranslatef(-345.0, 0.0, 0.0);
			/* save the current ListBase value, set it to 
			 * the start of the font and use the string itself
			 * to index into the display lists
			 */
			glPushAttrib(GL_LIST_BIT);
				glListBase(outlineBase);
				glCallLists(strlen(string), GL_UNSIGNED_BYTE, 
						(unsigned char *) string);
			glPopAttrib();
		glPopMatrix();

	glPopMatrix();

	auxSwapBuffers();
}
