/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* curve.c
 * This program draws a rotating NURBS curve.
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid initialize();
GLvoid drawScene( GLvoid );
GLvoid resize( GLsizei, GLsizei );

GLfloat ctlpoints[4][3];
GLUnurbsObj *theNurb;

main ()
{
	initialize();

	auxMainLoop( drawScene );

}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -5.0 ); 
}

GLvoid
initialize( )
{
	GLsizei  width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 2, 
		winSize / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "curve" );
	
	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -5.0 ); 

	/* Create NURBS object */
	theNurb = gluNewNurbsRenderer();

	/* Set longest line segment to something lower than
	 * the default to get smooth looking curves, should work
	 * well at maximum segment length of 10 pixels,
	 * but as of 5.1 alpha 151, bug caused need to set very small
	 */
#define NURBS_TOL_BUG 1
#ifdef NURBS_TOL_BUG
	gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 0.1);
#else
	gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 10.0);
#endif
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat angle = 0.0;
	int i;

	GLfloat curvePt[4][3] = 
	    	{{0.1, 0.5, 0.0}, {0.1, 0.9, 0.0}, 
		 {0.9, 0.9, 0.0}, {0.9, 0.5, 0.0}};

    	GLfloat curveKnots[8] = 
		{0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	glColor3f( 0.0, 1.0, 0.0 );
	glPushMatrix(); 
	
		glRotatef( angle, 0.0, 1.0, 0.0 );
		gluBeginCurve(theNurb);
			gluNurbsCurve (theNurb, 8, curveKnots, 3, 
				&curvePt[0][0], 4, GL_MAP1_VERTEX_3);
		gluEndCurve(theNurb);

	glPopMatrix();

	angle += 2.0;
	if (angle > 360.0)
		angle = 0.0;

	auxSwapBuffers();
}

