/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 * mask.c
 * This program draws a rotating nurbs surface shaped like a mask.
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>
/* rgbImageFile.h should be found in the ../../include directory */
#include "rgbImageFile.h"

/*  Function Prototypes  */

GLvoid  initialize();
GLvoid	drawScene( GLvoid );
GLvoid	resize( GLsizei width, GLsizei height );
GLvoid	initLighting( GLvoid );
GLvoid	initSurface( GLvoid );
GLvoid  initTexture( unsigned int *image, 
	GLsizei imageWidth, GLsizei imageHeight );
GLvoid  toggleTexture( GLvoid );

GLboolean textureFlag = GL_FALSE;
	
GLfloat ctlpoints[10][7][3] = 
{  /*   t = 0,         t = 1,        t = 2,        t = 3,        t = 4,        t = 5,        t = 6   */
{ {-1.4,7.1,0.0},{-1.2,7.1,0.2},{-1.0,7.1,0.3},{0.0,7.1,0.5},{1.0,7.1,0.3},{1.2,7.1,0.2},{1.4,7.1,0.0} }, /* s=0 */
{ {-1.6,7.0,0.0},{-1.4,7.0,0.7},{-1.0,7.0,0.9},{0.0,7.0,1.1},{1.0,7.0,0.9},{1.4,7.0,0.7},{1.6,7.0,0.0} }, /* s=1 */
{ {-1.8,4.5,0.0},{-1.5,4.5,0.5},{-1.0,4.5,0.7},{0.0,4.5,0.9},{1.0,4.5,0.7},{1.5,4.5,0.5},{1.8,4.5,0.0} }, /* s=2 */
{ {-1.9,2.6,0.0},{-1.5,2.6,1.1},{-1.0,2.6,1.0},{0.0,2.6,2.5},{1.0,2.6,1.0},{1.5,2.6,1.1},{1.9,2.6,0.0} }, /* s=3 */
{ {-1.8,2.8,0.0},{-1.5,2.8,0.8},{-1.0,2.8,0.9},{0.0,2.8,1.0},{1.0,2.8,0.9},{1.5,2.8,0.8},{1.8,2.8,0.0} }, /* s=4 */
{ {-1.8,1.8,0.0},{-1.5,1.8,0.8},{-1.0,1.8,1.0},{0.0,1.8,1.2},{1.0,1.8,1.0},{1.5,1.8,0.8},{1.8,1.8,0.0} }, /* s=5 */
{ {-1.8,1.6,0.0},{-1.5,1.6,0.7},{-1.0,1.6,0.8},{0.0,1.6,0.9},{1.0,1.6,0.8},{1.5,1.6,0.7},{1.8,1.6,0.0} }, /* s=6 */
{ {-1.7,0.9,0.0},{-1.5,0.9,0.8},{-1.0,0.9,1.0},{0.0,0.9,1.1},{1.0,0.9,1.0},{1.5,0.9,0.8},{1.7,0.9,0.0} }, /* s=7 */
{ {-1.5,.12,0.0},{-1.2,.12,0.6},{-1.0,.12,0.7},{0.0,.12,0.8},{1.0,.12,0.7},{1.2,.12,0.6},{1.5,.12,0.0} }, /* s=8 */
{ {-0.8,.15,0.0},{-1.1,.15,0.0},{-1.0,.15,0.0},{0.0,.15,0.0},{1.0,.15,0.0},{1.1,.15,0.0},{0.8,.15,0.0} } /* s=9 */
};

GLUnurbsObj *theNurb;

main ()
{
	initialize();
	auxMainLoop( drawScene );

}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 25.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -20.0 ); 
}

GLvoid
initialize( )
{
	GLsizei  width, height, winSize;
	char		*imageFileName = "cv.rgb";
	unsigned int 	*image;
	GLsizei		imageWidth, imageHeight;

	fprintf(stdout, "loading image %s\n", imageFileName );

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 2, winSize / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "mask" );
	
	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, toggleTexture );
	
	glClearColor( 0, 0, 0.8, 1 );
	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 25.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -20.0 ); 

	initLighting(); 

	/* enable automatic surface normal generation */
	glEnable( GL_AUTO_NORMAL );

	theNurb = gluNewNurbsRenderer();

	gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 50.0);

	glClear( GL_COLOR_BUFFER_BIT );
	auxSwapBuffers();

	image = rgbReadImageFile(imageFileName, &imageWidth,
			 &imageHeight);


	initTexture( image, imageWidth, imageHeight );
}

GLvoid initTexture( unsigned int *image, 
	GLsizei imageWidth, GLsizei imageHeight )
{
	/* scale texture image, make mipmaps and load texture
	 *	gluBuild2DMipmaps( target, components, width, height,
	 *		format,  type, imageArray ) 
	 */
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, imageWidth, imageHeight,
			 GL_RGBA, GL_UNSIGNED_BYTE, image);

						
	/* Decal uses only the texture color, not the original polygon 
	 * color, unless there is alpha in texture, in which case
	 * alpha determines the percentage of texture blended
	 * with the polygon color.
	 *
	 * Decal may be faster than the default modulate mode.
	 */
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	
	/* Set texture coordinate generation function to use 
	 * spherical coordinate mapping
	 */
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	toggleTexture();
}


/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{

	GLfloat mat_diffuse[] = { 1.0, 0.6, 0.0, 1.0 };
	GLfloat mat_emission[] = { 0.0, 0.0, 0.0, 1.0 };

	GLfloat light_position[] = { 1.0, 0.0, 1.0, 0.0 };

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, mat_emission);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid  toggleTexture( GLvoid )
{
	textureFlag = !textureFlag;
	if (textureFlag)
	{
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glEnable(GL_TEXTURE_2D);
	}
	else
	{
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_2D);
	}	
}


GLvoid
drawScene( GLvoid )
{
	static GLfloat angle = 0.0;
	GLfloat knots[8] = {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};
	GLfloat edgePt[5][2] = /* counter clockwise */
		{{0.0, 0.0}, {7.0, 0.0}, {7.0, 8.0}, {0.0, 8.0}, {0.0, 0.0}};

	GLfloat curveKnots[] = 
		{0.0, 0.0, 0.0, 0.0, 1.0, 2.0, 2.0, 2.0, 2.0};
	GLfloat leftEyeCurve[5][2] = /* clockwise */ 
		{{1.3, 3.0}, {1.3, 1.0}, {1.1, 1.0}, {1.1, 3.0}, {1.3, 3.0}};
	GLfloat rightEyeCurve[5][2] = /* clockwise */ 
		{{1.3, 5.0}, {1.1, 5.0}, {1.1, 7.0}, {1.3, 7.0}, {1.3, 5.0}};
	GLfloat s_knots[14] = 
	{ 
		0.0,0.0,0.0,0.0,
		1.0,2.0,3.0,4.0,5.0,6.0,
		7.0,7.0,7.0,7.0
	};

	GLfloat t_knots[11] = { 0.0,0.0,0.0,0.0,2.0,4.0,6.0,8.0,8.0,8.0,8.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix(); 
		glTranslatef( 0.0, -4.0, 0.0 );	
		glRotatef( angle, 0.0, 1.0, 0.0 );
		gluBeginSurface(theNurb);
			gluNurbsSurface(theNurb, 
				14, s_knots,
				11, t_knots,
				7 * 3, 3,
				&ctlpoints[0][0][0], 
				4, 4,
				GL_MAP2_VERTEX_3);
			gluBeginTrim (theNurb);
			    gluPwlCurve (theNurb, 5, &edgePt[0][0], 2,
					GLU_MAP1_TRIM_2);
			gluEndTrim (theNurb);

			gluBeginTrim (theNurb);
				gluNurbsCurve (theNurb, 9, curveKnots, 2, 
					&leftEyeCurve[0][0], 4, GLU_MAP1_TRIM_2);
			gluEndTrim (theNurb);
			gluBeginTrim (theNurb);
				gluNurbsCurve (theNurb, 9, curveKnots, 2, 
					&rightEyeCurve[0][0], 4, GLU_MAP1_TRIM_2);
			gluEndTrim (theNurb);
		gluEndSurface(theNurb);

	glPopMatrix();

	angle += 5.0;
	if (angle > 360.0)
		angle = 0.0;

	auxSwapBuffers();
}

