/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*
 * surface.c
 * This program draws a rotating NURBS surface.
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initialize();
GLvoid	drawScene( GLvoid );
GLvoid	resize( GLsizei width, GLsizei height );
GLvoid	initLighting( GLvoid );
GLvoid	initSurface( GLvoid );

GLfloat ctlpoints[4][4][3];
GLUnurbsObj *theNurb;

main ()
{
	initialize();
	auxMainLoop( drawScene );

}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -15.0 ); 
}

GLvoid
initialize( )
{
	GLsizei  width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 2, winSize / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "surface" );
	
	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 20.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -15.0 ); 

	initLighting(); 

	/* enable automatic surface normal generation */
	glEnable( GL_AUTO_NORMAL );

	theNurb = gluNewNurbsRenderer();

	/* Default edge length works well for surfaces */
	gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 50.0);

	initSurface();
}


/*  Initializes the control points of the surface to a small hill.
 *  The control points range from -3 to +3 in x, y, and z
 */
GLvoid initSurface(void)
{
    int s, t;

    for (s = 0; s < 4; s++) {
	for (t = 0; t < 4; t++) {
	    ctlpoints[s][t][0] = 2.0*((GLfloat)s - 1.5);
	    ctlpoints[s][t][1] = 2.0*((GLfloat)t - 1.5);

	    if ( (s == 1 || s == 2) && (t == 1 || t == 2))
		ctlpoints[s][t][2] = 3.0;
	    else
		ctlpoints[s][t][2] = -3.0;
	}
    }
}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{

	GLfloat mat_diffuse[] = { 0.8, 0.8, 0.2, 1.0 };
	GLfloat mat_specular[] = { 0.8, 0.0, 0.8, 1.0 };
	GLfloat mat_emission[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat mat_shininess[] = { 30.0 };

	GLfloat light_position[] = { 1.0, 0.0, 1.0, 0.0 };

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, mat_emission);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid
drawScene( GLvoid )
{
	static GLfloat angle = 0.0;

	GLfloat knots[8] = {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix(); 
	
		glRotatef( angle, 0.0, 1.0, 0.0 );
		gluBeginSurface(theNurb);
			gluNurbsSurface(theNurb, 
				8, knots,
				8, knots,
				4 * 3, 3,
				&ctlpoints[0][0][0], 
				4, 4,
				GL_MAP2_VERTEX_3);
		gluEndSurface(theNurb);

	glPopMatrix();

	angle += 5.0;
	if (angle > 360.0)
		angle = 0.0;

	auxSwapBuffers();
}

