/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				collage.c
 *	To run, type:
 *	collage <file.rgb> [<file2.rgb>] ...
 *
 *	Try "collage /usr/demos/data/textures/*.rgb"
 *
 *	Up to MAX_IMAGES can be read.  collage will randomly
 *	display images zoomed randomly in random positions.
 *		Escape key		- exit the program
 *		SPACEBAR		- toggle fixed/random zoom
 *
 *	David Marsland, MTS, SGI Education R & D, 1993
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <aux.h>
#include <math.h>
#include <stdio.h>
#include "rgbImageFile.h"

#define MAX_IMAGES 100

/*  Function Prototypes  */

GLvoid  initialize( int argc, char *argv[] );
GLvoid  drawScene( GLvoid );
GLvoid  printHelp( GLvoid );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  zoomToggle( GLvoid );

/* Global Variables */
unsigned int *images[MAX_IMAGES] = { NULL };
int imageWidths[MAX_IMAGES], imageHeights[MAX_IMAGES];
GLsizei     screenWidth, screenHeight;
GLsizei     windowWidth, windowHeight;
GLint       xPos, yPos;
GLint       imageCount;
GLboolean   zoomFlag = GL_FALSE;

main ( int argc, char *argv[])
{
	if (argc < 2) {
		fprintf (stderr, 
		  "usage: collage <imageFile1> [<imageFile2> ... ]\n");
		exit (1);
	}
	initialize( argc, argv );

	auxMainLoop( drawScene );
}

GLvoid  
zoomToggle( GLvoid )
{
	zoomFlag = !zoomFlag;
	if ( !zoomFlag)
		glPixelZoom( 1.0, 1.0 );
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLint   matrixMode;

	windowWidth = width;
	windowHeight = height;

	glGetIntegerv( GL_MATRIX_MODE, &matrixMode );
	glLoadIdentity();
	glOrtho( -0.5, (GLfloat) windowWidth-0.5,
		-0.5, (GLfloat) windowHeight-0.5, -1.0, 1.0 );
	glViewport( 0, 0, windowWidth - 1, windowHeight - 1);
	glMatrixMode( matrixMode );
	glLoadIdentity();
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, "\n\
collage creates a collage of images which can be zoomed\n\n\
Escape key		- exit the program\n\
SPACEBAR		- toggle fixed/random zoom\n\n");
}

GLvoid
initialize( int argc, char *argv[] )
{
	int i;

	imageCount = argc - 1;
	if (imageCount > MAX_IMAGES)
		imageCount = MAX_IMAGES;

	srand48( imageCount );

	/* read in all image files specifed as command line args */
	for ( i = 0; i < imageCount; i++ )
	{
		images[i] = rgbReadImageFile( argv[i+1],
			&imageWidths[i], &imageHeights[i]);
	}

	auxGetScreenSize( &screenWidth, &screenHeight );
	auxInitPosition( screenWidth/8, screenHeight/8, 
		3*screenWidth/4, 3*screenHeight/4 );
	auxInitDisplayMode( AUX_RGBA | AUX_SINGLE );
	auxInitWindow( "collage     SPACEBAR" );
	auxGetSize( &windowWidth,  &windowHeight );

	auxAnimation( AUX_ON );
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClear( GL_COLOR_BUFFER_BIT );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	auxKeyFunc( AUX_SPACE, zoomToggle );

	auxReshapeFunc( resize );

	glMatrixMode( GL_PROJECTION );
	glOrtho( -0.5, (GLfloat) windowWidth-0.5,
		-0.5, (GLfloat) windowHeight-0.5, -1.0, 1.0 );
	glViewport( 0, 0, windowWidth - 1, windowHeight - 1);
	glMatrixMode( GL_MODELVIEW );

	printHelp();
}


GLvoid
drawScene( GLvoid )
{
	static int i;

	i = (int) (imageCount * drand48());

	if ( zoomFlag )
	{
		glPixelZoom( ( drand48() * 5.0 ) - 1.5, 
				( drand48() * 5.0 ) - 1.5 );
	}
	
	xPos += (2.0*drand48() - 1.0)*.05*screenWidth;
	yPos += (2.0*drand48() - 1.0)*.05*screenHeight;
	xPos = fabs(xPos);
	yPos = fabs(yPos);
	glRasterPos2f(fmodf(xPos, windowWidth), 
			fmodf(yPos, windowHeight) );
	glDrawPixels (imageWidths[i], imageHeights[i],
			GL_RGBA, GL_UNSIGNED_BYTE, images[i]);
	glFlush();
}
