/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				
 *  copyZoom.c - demonstates the glCopyPixels() and glPixelZoom() 
 *	routines.
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initialize( GLvoid );
GLvoid	printHelp( GLvoid );
GLvoid	drawScene( GLvoid );
GLvoid	resize( GLsizei width, GLsizei height );
GLvoid	initLighting( GLvoid );
GLvoid  leftMouseDown( AUX_EVENTREC * );
GLvoid  copyZoom( GLvoid );
GLvoid  redRectangle( GLvoid );
GLvoid  zoomUp( AUX_EVENTREC * );
GLvoid  zoomDown( AUX_EVENTREC * );

/* Global Variables */
GLsizei	windowWidth, windowHeight;
GLfloat zoomFactor = 2.0;
GLint   xPos, yPos, midX;
GLint   xToCopy,  yToCopy;
GLsizei widthToCopy, heightToCopy;

main ()
{
	
	initialize();

	auxMainLoop( drawScene );

}

GLvoid
initialize( GLvoid )
{
	GLsizei width, height, winSize;

	printHelp();
		
	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	windowWidth = (winSize & 0x1 ? winSize : winSize - 1);
	midX = windowWidth/2;
	windowHeight = winSize/2;
	xPos = yPos = winSize/4;
	auxInitPosition( xPos, yPos, windowWidth, windowHeight );

	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "copyZoom       LEFT/MIDDLE/RIGHT -MOUSE" );
	
	auxReshapeFunc( resize );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, leftMouseDown );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, zoomDown );
	auxMouseFunc( AUX_RIGHTBUTTON, AUX_MOUSEDOWN, zoomUp );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	glClearColor( 0, 0, 0, 1 );
	glEnable( GL_DEPTH_TEST );

	/* Create the first viewport - the left half of the window */
	glViewport( 0, 0, midX + 1, windowHeight );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -8.0 );

	initLighting(); 
}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 20.0 };

	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, 
"\ncopyZoom, copies and zooms pixels under the cursor  \n\
when Left Mousebutton is pressed\n\n\
Escape key			- exit the program\n\
Left Mousebutton, down		- copy pixels\n\
Middle Mousebutton, down	- zoom down\n\
Right Mousebutton, down		- zoom up\n\n");
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	windowWidth = (width & 0x1 ? width : width - 1);
	midX = windowWidth/2;
	windowHeight = height;
	glViewport( 0, 0, midX + 1, height );

	aspect = (GLfloat) (width/2.0) / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0 ); 
}

	
GLvoid
leftMouseDown( AUX_EVENTREC *event )
{
	xPos = event->data[AUX_MOUSEX];
	yPos = windowHeight - event->data[AUX_MOUSEY];
}

GLvoid
zoomUp( AUX_EVENTREC *event )
{
	zoomFactor *= 2;
	printf("Zoom Factor = %g\n", zoomFactor);
}

GLvoid
zoomDown( AUX_EVENTREC *event )
{
	zoomFactor /= 2;
	printf("Zoom Factor = %g\n", zoomFactor);
}

/*
 * redRectangle() calculates size of area to be copied and 
 *	outlines it in red.
 */
GLvoid
redRectangle( GLvoid )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	/* set up projection so world xy maps directly to window xy */
	glOrtho( -0.5, midX + 0.5, -0.5, windowHeight - 0.5, 1.0, -1.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	/* calculate size of area to copy to the right half of the
	 * window
	 */
	xToCopy = xPos - (midX-2)/(2.0*zoomFactor); 
	yToCopy = yPos - (windowHeight-1)/(2.0*zoomFactor); 
	widthToCopy = (midX-2)/zoomFactor; 
	heightToCopy = (windowHeight-1)/zoomFactor; 

	glColor3f( 1.0,  0.0,  0.0 );
        glBegin( GL_LINE_LOOP );
        	glVertex2i( xToCopy, yToCopy );
        	glVertex2i( xToCopy + widthToCopy, yToCopy );
        	glVertex2i( xToCopy + widthToCopy, yToCopy + heightToCopy );
        	glVertex2i( xToCopy, yToCopy + heightToCopy );
        glEnd();
	glFlush();
}

/*
 * copyZoom() copies pixels from viewport on left half of window, 
 * zooming by zoomFactor.
 */
GLvoid
copyZoom( GLvoid )
{
	/* Create the second viewport - the right half of the window */
	glViewport( midX + 1, 0, midX - 1, windowHeight );

	/* adjust for values out of the viewport */
	if (xToCopy < 0) {
		widthToCopy += xToCopy;
		xToCopy = 0; 
	}
	if (xToCopy + widthToCopy > windowWidth/2) {
		widthToCopy -= (xToCopy + widthToCopy) - windowWidth/2;
	}
	if (yToCopy < 0) {
		heightToCopy += yToCopy;
		yToCopy = 0;
	}
	if (yToCopy + heightToCopy > windowHeight) {
		heightToCopy -= (yToCopy + heightToCopy + 1) - windowHeight;
	}

	/* copy pixels */
	glPixelZoom( zoomFactor, zoomFactor );
	glRasterPos2f(0, 0);
	glCopyPixels (xToCopy, yToCopy, widthToCopy + 1, heightToCopy + 1, GL_COLOR );
}

GLvoid
drawScene( GLvoid )
{
	int i,  slices = 8;
	static GLfloat spin = 0.0;

	/* Create the first viewport - the left half of the window */
	glViewport( 0, 0, midX + 1, windowHeight );

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	glEnable( GL_LIGHTING );
	glEnable( GL_DEPTH_TEST );

	glPushMatrix();
	glRotatef( spin, 0, 1, 0 );

	for ( i = 0; i < slices; i++ )
	{
		glColor3f( i/10.0, i/10.0, 1.0 - i/10.0 ); 
		glPushMatrix(); 
			glRotatef( i * 360.0/slices, 0, 0, 1 );
			glTranslatef( 1.5, 0.0, 0.0 );
			glRotatef( i * 360.0/slices, 0, 1, 0 );
			auxSolidTorus( 0.25, 0.75 );
		glPopMatrix();
	}
	/* use glFinish() to ensure that image is rendered 
	 *  before copyZoom() is called
	 */
	glDisable( GL_DEPTH_TEST );
	glDisable( GL_LIGHTING );

	glPopMatrix();
	glFinish();

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();

		glMatrixMode( GL_MODELVIEW );
		glPushMatrix();

			/* draw red box around image to zoom */
			redRectangle();

			/* draw white divider line between view halves */
			glColor3f( 1.0,  1.0,  1.0 );
			glBegin( GL_LINES );
				glVertex2i( midX, 0 );
				glVertex2i( midX, windowHeight);
			glEnd();

			copyZoom();

			glMatrixMode( GL_PROJECTION );
		glPopMatrix();

		glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
	
	auxSwapBuffers();
	spin = fmodf( spin + 2.0, 360.0 );
}
