/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				show.c
 *	Type "show <imageFileName>" to create a window displaying any 
 *      RGB image stored in the SGI .rgb format.
 *
 *	The filename is opened and the image data is read.  
 *	The image is then displayed in a window.
 *
 *	Note that this program must be linked with -lrgbImageFile, 
 *	which should be in the ../../librgb directory.
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>
/* rgbImageFile.h should be found in the ../../include directory */
#include "rgbImageFile.h"

/*  Function Prototypes  */

GLvoid  initialize( int argc, char *argv[] );
GLvoid  drawScene( GLvoid );
GLvoid  resize( GLsizei, GLsizei );

/* Global Variables */
unsigned int *image;
int width, height;

main ( int argc, char *argv[])
{
	if (argc < 2) {
		fprintf (stderr, "usage:  show <imageFileName>\n");
		exit (1);
	}
	
	initialize( argc,  argv );

	auxMainLoop( drawScene );
}


GLvoid
initialize( int argc, char *argv[] )
{
	image = rgbReadImageFile(argv[1], &width, &height);

	auxInitPosition ( 0, 0, width, height );

	auxInitDisplayMode( AUX_RGBA | AUX_SINGLE );

	auxInitWindow( "show" );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	auxReshapeFunc( resize );

	glMatrixMode( GL_PROJECTION );
	glOrtho( -0.5, (GLfloat) width-0.5,
		-0.5, (GLfloat) height-0.5, -1.0, 1.0 );
	glMatrixMode( GL_MODELVIEW );

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	glViewport( 0, 0, width, height );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( -0.5, (GLfloat) width-0.5,
		-0.5, (GLfloat) height-0.5, -1.0, 1.0 );
	glMatrixMode( GL_MODELVIEW );
}

GLvoid
drawScene( GLvoid )
{
	glClear( GL_COLOR_BUFFER_BIT );

	/* Set raster position.  Note that glRasterPos2i() takes
	 * world coordinates, which are transformed by the current 
	 * MODELVIEW and PROJECTION matrices.
	 *
	 * In this example, both contain Identity matrices, 
	 * causing these xy coordinates to map directly to window
	 * coordinates.  This would not be the case if any
	 * projection or modeling transforms had been applied.
	 */
	glRasterPos2i( 0,  0 );
	glDrawPixels (width, height, GL_RGBA, GL_UNSIGNED_BYTE, image);
	
	glFlush();
}
