/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				windowSnap.c
 *	Type "windowSnap <imageFileName>" When the leftmouse 
 *	button is pressed and released the window is captured,
 *	and the pixel data is written into a file.  
 *
 *	Note that this program must be linked with -lrgbImageFile, 
 *	which should be in the ../../librgb directory.
 */

#include <aux.h>
#include <math.h>
#include <stdio.h>
/* rgbImageFile.h should be found in the ../../include directory */
#include "rgbImageFile.h"

/*  Function Prototypes  */
GLvoid  initialize( GLvoid );
GLvoid	printHelp( char * );
GLvoid	grabImage( AUX_EVENTREC * );
GLvoid	drawScene( GLvoid );
GLvoid	resize( GLsizei width, GLsizei height );
GLvoid	initLighting( GLvoid );

/* Global Variables */
unsigned int *image;
GLsizei width, height;
char *imageFileName;

main ( int argc, char *argv[] )
{
	if (argc < 2) {
		fprintf(stderr, "usage:  windowSnap <imageFileName>\n");
		imageFileName = "foo.rgb";
	} else 
		imageFileName = argv[1];
	fprintf(stdout, "Output will be saved in %s\n", 
		imageFileName);
	
	initialize();	
	printHelp( argv[0] );
	auxMainLoop( drawScene );
}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0 ); 
}

GLvoid
initialize( GLvoid )
{
	GLsizei winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition(winSize/4, winSize/4, winSize/2, winSize/2);
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH );
	auxInitWindow( "windowSnap" );

	auxReshapeFunc( resize );

	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, grabImage );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	glClearColor( 0, 0, 0, 1 );

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -8.0 );

	initLighting(); 
}


/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{
	/*	mat_specular and mat_shininess are NOT default values	*/
	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };

	GLfloat light_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
	GLfloat light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };

	/*	light_position is NOT default value	*/
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
}

GLvoid
printHelp( char *progname )
{
	fprintf(stdout, 
	"\n%s, reads pixels from window and saves in file \n"\
	"when the Left Mousebutton is released\n\n"\
	"Escape key               - exit the program\n"\
	"Left Mousebutton, up     - snapshot of window\n\n", 
	progname);
}

GLvoid
grabImage( AUX_EVENTREC *event )
{
	auxGetSize( &width, &height ); 

	/* allocate space for the pixel data we're going to read */
	image = (unsigned int *)
		malloc(width * height * sizeof( unsigned int ) );

	glReadPixels ( 0, 0, width, height, GL_RGBA, 
		GL_UNSIGNED_BYTE, image );

	rgbWriteImageFile( imageFileName, width, height, image );
	
	auxQuit();
}


GLvoid
drawScene( GLvoid )
{
	int i,  slices = 8;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	for ( i = 0; i < slices; i++ )
	{
		glColor3f( i/10.0, i/10.0, 1.0 - i/10.0 ); 
		glPushMatrix(); 
		glRotatef( i * 360.0/slices, 0, 0, 1 );
		glTranslatef( 1.5, 0.0, 0.0 );
		glRotatef( i * 360.0/slices, 0, 1, 0 );
		auxSolidTorus( 0.25, 0.75 );
		glPopMatrix();
	}
	glFlush();
}

