/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/*				stencil.c
 *  This program demonstrates a simple application of the stencil
 *  planes to mask an arbitrary screen area.
 */

#include "aux.h"
#include <math.h>
#include <stdio.h>

/*  Function Prototypes  */

GLvoid  initialize();
GLvoid	drawScene( GLvoid );
GLvoid	resize( GLsizei width, GLsizei height );
GLvoid	initLighting( GLvoid );
GLvoid 	setupStencil( GLvoid );

main ()
{
	initialize();

	auxMainLoop( drawScene );

}

GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat    aspect;

	glViewport( 0, 0, width, height );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	aspect = (GLfloat) width / height;
	gluPerspective( 45.0, aspect, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -5.0 ); 

	/* reinitialize stencil plane to fit new window size */
	setupStencil();
}

GLvoid
initialize( )
{
	GLsizei  width, height, winSize;

	auxGetScreenSize( &width, &height );
	winSize = ( width < height ) ? width : height;
	auxInitPosition( winSize / 4, winSize / 4, winSize / 2, winSize / 2 );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE | AUX_STENCIL );
	auxInitWindow( "stencil" );
	
	auxReshapeFunc( resize );
	auxAnimation( AUX_ON );

	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	glClearColor( 0, 0, 0, 1 );

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	gluPerspective( 45.0, 1.0, 3.0, 13.0 );
	glMatrixMode( GL_MODELVIEW );
	glTranslatef( 0.0, 0.0, -5.0 );

	initLighting(); 

	glClearStencil(0);
	glStencilMask(1);

	glEnable(GL_STENCIL_TEST);

	setupStencil();
}

/*  Initialize material property, light source, lighting model, 
 *  and depth buffer.
 */
GLvoid
initLighting( GLvoid )
{

	GLfloat mat_specular[] = { 0.8, 0.8, 0.8, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glShadeModel( GL_SMOOTH );

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}



/* setupStencil() creates a simple diamond-shaped stencil
 * pattern stored in the stencil planes.  This sets up the
 * stencil for use in drawScene().
 */
GLvoid
setupStencil( GLvoid )
{
	glClear(GL_STENCIL_BUFFER_BIT);
	glStencilFunc(GL_ALWAYS, 1, 1);
	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);

	glColorMask(0, 0, 0, 0);
	glDepthMask(0);
	glBegin(GL_QUADS);        
	    glVertex3i( 1, 0, 0 );
	    glVertex3i( 0, 1, 0 );
	    glVertex3i( -1, 0, 0 );
	    glVertex3i( 0, -1, 0 );
	glEnd();
	glDepthMask(1);
	glColorMask(1, 1, 1, 1);
}
    

GLvoid
drawScene( GLvoid )
{
	static GLfloat angle = 0.0;
	GLfloat yellow[] = { 1.0, 1.0, 0.0, 1.0 };
	GLfloat magenta[] = { 1.0, 0.0, 1.0, 1.0 };

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

	glPushMatrix(); 
		glRotatef( angle, 0.0, 1.0, 0.0 );

		/* only draw where the stencil bits == 1 */
		glStencilFunc(GL_EQUAL, 1, 1);
		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

		/* draw yellow torus */
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, yellow);
		auxSolidTorus( 0.25, 0.75 );

		/* only draw where the stencil bits == 0 */
		glStencilFunc(GL_EQUAL, 0, 1);
		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
	    
		/* draw magenta torus */
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, magenta);
		auxSolidTorus( 0.25, 0.75 );
	glPopMatrix(); 

	auxSwapBuffers();

	angle = fmodf( (angle + 1.0), 360.0 );

}

