/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
 *  flyutil.c 
 *  This file contains utility routines used by the fly program.
 *
 *  Copyright 1991, 1992, 1993, 1994, Silicon Graphics,  Inc.
 *  Technical Education R & D
 */

#include "fly.h"

/* Global Variables */

GLint       drawStyle = LIT_SMOOTH;
GLboolean   drawNormalsFlag = GL_FALSE;

/* initial characteristics of the eye */
GLint direction = FORWARD; 		/* FORWARD or BACKWARD */
extern GLboolean moving;
extern GLboolean autopilot;		/* is autopilot mode on? */
extern GLboolean debugFlag;		/* print eye position info? */

GLfloat     grid[GRIDPOINTS+1][GRIDPOINTS+1][3] = { 0 };
GLfloat     gridNormals[GRIDPOINTS+1][GRIDPOINTS+1][3] = { 0 };

extern GLsizei     winWidth, winHeight;

GLint xPos,yPos;		/* mouse position */

float ex, ey, ez;		/* eye position */
GLfloat dx,dy,dz;		/* change in eye position */
float yaw;			/* eye rotation about y axis in degrees */
float pitch;			/* eye rotation about x axis in degrees */		
float velocity = 0.15;		/* eye velocity */
float turn_rate = 3.0;		/* eye turning agility */

GLvoid
initializeEye(GLvoid)
{
	/* initial position of the eye in the world */
	ex = 20.0;
	ey = 10.0;
	ez = 20.0;

	yaw = -45.0;		/* rotation around eye y axis in degrees */
	pitch = 20.0;		/* rotation around eye x axis in degrees */
	autopilot = GL_FALSE;
	moving = GL_FALSE;
}


/* Define user input functions */
GLvoid  
toggleDrawNormals( GLvoid )
{
	drawNormalsFlag = !drawNormalsFlag;
}

GLvoid
setDebug( GLvoid )
{
	debugFlag = !debugFlag;
}

GLvoid
setDrawStyle( GLvoid )
{
	if (++drawStyle > LIT_SMOOTH)
		drawStyle = WIRE;

	switch (drawStyle)
	{
		case WIRE:
			break;
		case LIT_FLAT:
			glShadeModel( GL_FLAT );
			break;
		case LIT_SMOOTH:
			glShadeModel( GL_SMOOTH );
			break;
		default:
			break;
	}
}

GLvoid
setAutopilot( GLvoid )
{
	autopilot = ! autopilot;
	if ( autopilot )
	{
		moving = GL_FALSE; 
	}
}

GLvoid
startMove( AUX_EVENTREC *event )
{
	auxMouseFunc( AUX_MOUSELOC, NULL, updatePosition );
	moving = GL_TRUE;
	direction = FORWARD;
	autopilot = GL_FALSE;
	updatePosition( event );
}

GLvoid
stopMove( AUX_EVENTREC *event )
{
	auxDeleteMouseFunc( AUX_MOUSELOC, NULL, updatePosition );
	moving = GL_FALSE;
}

GLvoid
startMoveBack( AUX_EVENTREC *event )
{
	auxMouseFunc( AUX_MOUSELOC, NULL, updatePosition );
	moving = GL_TRUE;
	direction = BACKWARD;
	autopilot = GL_FALSE;
	updatePosition( event );
}

GLvoid
stopMoveBack( AUX_EVENTREC *event )
{
	auxDeleteMouseFunc( AUX_MOUSELOC, NULL, updatePosition );
	moving = GL_FALSE;
}

GLvoid  
moveEye( GLvoid )
{
	static float auto_angle;	/* autopilot rotation */
	
	if (autopilot)
	{
		/* using autopilot, eye moves in a circle */
		ex = 10.0*fcos(auto_angle);
		ey = 2.0;
		ez = 10.0*fsin(auto_angle);
	
		/* increment the angle around the circle */
		auto_angle = auto_angle + velocity/10.0;
	
		/* update the rotation of the eye, always look forward */
		yaw = 180.0/M_PI*auto_angle + 180.0;
		
		if (debugFlag)
			fprintf(stdout,"\nex=%f,ey=%f,ez=%f\n",ex,ey,ez);
	}
	/* else if not using the autopilot, calculate new 
	 * eye position based on the mouse position 
	 */
	else if (moving)
	{
		/* update eye orientation */
		yaw += turn_rate*(float)(xPos-winWidth/2)/winWidth;
		pitch += turn_rate*(float)(yPos-winHeight/2)/winHeight;
	
		/* changes in eye position */
		dx = fsin(yaw  * M_PI/180.0);
		dy = fsin(pitch* M_PI/180.0);
		dz = fcos(yaw  * M_PI/180.0);
	
		/* update eye position */
		ex = ex + velocity*direction*dx;
		ey = ey - velocity*direction*dy;
		ez = ez - velocity*direction*dz;
		if (debugFlag)
			fprintf (stdout, "\nex=%f, ey=%f, ez=%f\n\
dx=%f, dy=%f, dz=%f\n\ yaw=%f,  pitch=%f\n", ex, ey, ez, dx, dy, dz, yaw, pitch);
	}   
}

/* Updates the eye position and orientation based on the position 
 * of the mouse. Works somewhat like the stick on an airplane:
 * Pushing forward on the mouse tilts down, pulling back tilts up.
 */
GLvoid  
updatePosition( AUX_EVENTREC *event )
{

	if ( moving ) 
	{
		/* mouse position in window coords */
		xPos = event->data[AUX_MOUSEX];
		/* compensate for difference in y origin */
		yPos = winHeight - event->data[AUX_MOUSEY];
	}
}


GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;

	glViewport( 0, 0, width, height );
	winWidth = width;
	winHeight = height;
	
	aspect = (GLfloat) width / height;

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 0.02, 100.0 );
	glMatrixMode( GL_MODELVIEW );
}

GLvoid
computeGrid( GLvoid )
{
	register int i, j;
	GLfloat     x, y, z, d, spacing, maxDist;
	static GLfloat angle;

	/* Create an undulating grid composed of points where the y
	 * value is related to the distance from the origin multiplied
	 * by the sine of an angle that is increasing by an amount
	 * proportional to velocity each iteration.
	 */

	/* spacing = distance between adjacent grid points in x & y */
	spacing = 20.0 / (float)(GRIDPOINTS - 1);

	/* farthest a gridpoint can be from the origin */
	maxDist = sqrtf( 200.0 );

	/* x and z range from -10.0 to 10.0 by spacing interval */
	/* We are going to GRIDPOINTS+1 in i and j so that we'll 
	 * be able to compute the normals later.
	*/
	for ( x = -10.0, i = 0; i < GRIDPOINTS+1; x += spacing, i++ ) {
		for ( z = -10.0, j = 0; j < GRIDPOINTS+1; z += spacing, j++ ) {
			/* compute d = distance from center of grid */
			d = sqrtf( x*x + z*z );
			y = 2.5*(maxDist-d)/maxDist*sinf((d+angle)*(360.0/10.0)*M_PI/180.0);
			grid[i][j][X] = x;
			grid[i][j][Y] = y;
			grid[i][j][Z] = z;
		}
	}
	angle = fmodf (angle + 0.4*velocity, 360.0);
        computeNormals();
} /* computeGrid */

GLvoid
drawGrid( GLvoid )
{
	GLfloat gridDiffuse[] = { 0.05, 0.15, 1.0, 1.0 };

	glMaterialfv(GL_FRONT, GL_AMBIENT, gridDiffuse);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, gridDiffuse);
	if (drawStyle == WIRE)
		drawGridWire();
	else
		drawGridFilled();
}

GLvoid
drawGridWire( GLvoid )
{
	register int i, j;

	/* draw vertical lines connecting grid points */
	for ( i = 0; i < (GRIDPOINTS); i++ ) {
		glBegin( GL_LINE_STRIP );
		for ( j = 0; j < (GRIDPOINTS); j++ ) {
			glNormal3fv( &gridNormals[i][j][0] );
			glVertex3fv( &grid[i][j][0] );
		}
		glEnd();
	}

	/* draw horizontal lines connecting grid points */
	for ( j = 0; j < (GRIDPOINTS); j++ ) {
		glBegin( GL_LINE_STRIP );
		for ( i = 0; i < (GRIDPOINTS); i++ ) {
			glNormal3fv( &gridNormals[i][j][0] );
			glVertex3fv( &grid[i][j][0] );
		}
		glEnd();
	}
} /* drawGridWire */

GLvoid
drawGridFilled( GLvoid )
{
	register int i, j;

	/* Draw quad strips connecting grid points. 
	 * Grid is drawn column by column, 
	 * where each column is one quad strip.
	 * Every pair of vertices after first two vertices
	 * adds a new quad to the strip.
	 */
	for ( i = 0; i < (GRIDPOINTS - 1); i++ ) {
		glBegin( GL_QUAD_STRIP );
		for ( j = 0; j < (GRIDPOINTS); j++ ) {
			glNormal3fv( &gridNormals[i][j][0] );
			glVertex3fv( &grid[i][j][0] );
			glNormal3fv( &gridNormals[i + 1][j][0] );
			glVertex3fv( &grid[i + 1][j][0] );
		}
		glEnd();
	}
	if (drawNormalsFlag)
		drawNormals();
}

/* Compute normals at each vertex of the grid,
 * computed by taking the cross products of two
 * points on the surface, and normalizing.
 */
GLvoid
computeNormals( GLvoid )
{
	register int i, j;
	float dx,dy,dz,l;       
	float ax,ay,az;
	float bx,by,bz;
	float nx,ny,nz;

	for (i=0; i < GRIDPOINTS; i++) {
		for ( j=0; j < GRIDPOINTS; j++) {

			/* Compute the direction vectors of two vectors
			 * in the polygon.  Let these two vectors be such
			 * that aXb points up (has positive y), so when we
			 * look at the grid from above, the grid is lit.
			 */
			ax=grid[i][j+1][X]-grid[i][j][X];
			ay=grid[i][j+1][Y]-grid[i][j][Y];
			az=grid[i][j+1][Z]-grid[i][j][Z];
			bx=grid[i+1][j][X]-grid[i][j][X];
			by=grid[i+1][j][Y]-grid[i][j][Y];
			bz=grid[i+1][j][Z]-grid[i][j][Z];

			/* compute the cross product of those two
			 * direction vectors 
			 */
			nx = ay*bz-az*by;
			ny = az*bx-ax*bz;
			nz = ax*by-ay*bx;

			/* normalize the cross product by dividing it by
			 * it's length 
			 */
			l=sqrtf(nx*nx+ny*ny+nz*nz);
			gridNormals[i][j][X] = nx/l;
			gridNormals[i][j][Y] = ny/l;
			gridNormals[i][j][Z] = nz/l;
		}
	}
} /* computeNormals */

/* Draw lines indicating normals at each vertex of the grid,
 * computed by taking the cross products of two
 * points on the surface, and normalizing.
 */
GLvoid
drawNormals( GLvoid )
{
	register int i, j;
	float dx,dy,dz,l;       
	float ax,ay,az;
	float bx,by,bz;
	float nx,ny,nz;
	float nv[3];
	static GLfloat    redColor[] = { 1.0, 0.0, 0.0 };

	glPushAttrib( GL_ENABLE_BIT );
		glDisable(GL_LIGHTING);
		glColor3fv( redColor );

		glBegin( GL_LINES );
		for (i=0; i < GRIDPOINTS; i++) {
			for ( j=0; j < GRIDPOINTS; j++) {
				nv[X]=grid[i][j][X]+gridNormals[i][j][X];
				nv[Y]=grid[i][j][Y]+gridNormals[i][j][Y];
				nv[Z]=grid[i][j][Z]+gridNormals[i][j][Z];
				glVertex3fv ( grid[i][j]);
				glVertex3fv ( nv );
			}
		}
		glEnd();
	glPopAttrib();
} /* drawNormals */
