/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
 * superfly.c This program allows the user to "fly" through a 
 *	simple virtual world. 
 *
 * Sample solutions for opengl2 labs.
 * Your challenge - make an even cooler fly!
 *
 * Utility routines are located in flyutil.c
 *
 * MODIFYING THE PROGRAM
 * ---------------------
 * New shapes should be added to the world in the routine
 * drawShapes(). 
 *
 * If new shapes are to be on the path of the autopilot,
 * they must be near a circle centered at the world 
 * origin with a radius of 10.0, parallel to and
 * raised above the 'floor,' the X-Z plane.
 *
 * As a starting point, here are some good places to center your
 * objects:
 *		( 0.0, 1.0, 10.0 )
 *		( 7.0, 1.0, 7.0  )
 *		( 10.0, 1.0, 0.0 )
 *		( 7.0, 1.0, -7.0 )
 *		( 0.0, 1.0, -10.0)
 *		( -7.0, 1.0, -7.0)
 *		( -10.0, 1.0, 0.0)
 *		( -7.0, 1.0, 7.0 )
 *
 *  Copyright 1991, 1992, 1993, 1994, Silicon Graphics,  Inc.
 *  Technical Education Development
 */


/* fly.h contains function prototypes, defines constants, and */
/* includes GL/gl.h, GL/glu.h, math.h, stdio.h, and aux.h */
#include "fly.h"  
#include "rgbImageFile.h"  

/* function prototypes for functions added to original fly.c */
GLvoid toggleRaster( GLvoid );
GLvoid toggleTexture( GLvoid );
GLvoid startPicking( AUX_EVENTREC *event );
GLvoid printHits (GLint hits, GLuint buffer[] );
GLvoid cullingHits (GLint hits, GLuint buffer[] );
GLvoid cullShapes( GLvoid );
GLvoid drawSmallWorld( GLvoid );
GLvoid toggleBlend( GLvoid );
GLvoid toggleCulling( GLvoid );
GLvoid toggleFog( GLvoid );
GLvoid toggleMovingClip( GLvoid );
GLint  initTextureList( char *imageFileName );
GLvoid incDensity( GLvoid );
GLvoid decDensity( GLvoid );
GLvoid resetEye( GLvoid );
GLvoid enableBlending( GLvoid );
GLvoid disableBlending( GLvoid );

/* Global Variables */

enum shapes { GRID, TETRA, CUBE, OCTA, DODECA, ICOSA, CONE,
	CYLINDER, SPHERE, TORUS };
enum drawModes { WIREFRAME, SOLID, SIMPLE };
enum clipModes { OFF, ATTACH_EYE, FIXED_IN_SCENE, MOVING };
GLboolean pickedFlag[ TORUS + 1 ] = { GL_FALSE };
GLboolean visibleFlag[ TORUS + 1 ] = { GL_FALSE };
char *shapeNames[] = 
{ "grid", "tetra", "cube", "octa", "dodeca", "icosa", "cone",
	"cylinder", "sphere", "torus" };

GLsizei     winWidth, winHeight;
GLboolean moving = GL_FALSE;
GLboolean autopilot = GL_FALSE;		/* is autopilot mode on? */
GLboolean debugFlag = GL_FALSE;		/* print eye position info? */
GLboolean cullingFlag = GL_FALSE;	/* in culling mode? */
GLint textureList;
GLboolean calledRecursively = GL_FALSE;

GLboolean clipMode = OFF;

extern GLint drawStyle;

extern float ex, ey, ez;	/* eye position */
extern float yaw;		/* eye rotation about y axis in degrees */
extern float pitch;		/* eye rotation about x axis in degrees */		
extern float velocity;		/* eye velocity */

unsigned int *image;
int imageWidth, imageHeight;

GLboolean picking = GL_FALSE;
GLboolean culling = GL_FALSE;
GLboolean blendFlag = GL_FALSE;
GLboolean rasterFlag = GL_FALSE;
GLboolean textureFlag = GL_FALSE;
GLboolean fogFlag = GL_FALSE;
GLfloat	  fogDensity = 0.05;

#define BUFSIZE 1024

GLvoid
main()
{
	initialize();
	auxMainLoop( drawScene );
}

GLvoid
printHelp( GLvoid )
{
	fprintf(stdout, 
"\nsuperfly.c - This program allows the user to fly through a \n" 
"simple virtual world.\n"
"<a> key	- <a>utopilot start/stop \n"
"<b> key	- <b>lend, alpha blending \n"
"<c> key	- cycle <c>lipping plane modes \n"
"<d> key	- <d>ebug mode, prints coords \n"
"<f> key	- <f>og toggle \n"
"<i> key	- <i>mage, displays raster image \n"
"<n> key	- <n>ormals drawing toggle \n"
"<r> key	- <r>eset eye position \n"
"<s> key	- <s>election culling \n"
"<t> key	- <t>exture mapping toggle \n"
"SPACE key	- cycle wireframe/flat shaded/smooth \n"
"LEFT Mouse 	- control moving the eye forward \n"
"MIDDLE Mouse 	- control moving the eye backward \n"
"Mouse pointer	- change direction: pull back to go up, forward to dive\n"
"RIGHT Mouse	- pick a shape\n"
"Escape key	- exit the program \n"
"<?> key 	- help, prints this info \n\n" );
}

GLvoid
initialize( GLvoid )
{
	GLfloat aspect;
	GLsizei screenWidth, screenHeight, winSize;
	
	auxGetScreenSize( &screenWidth, &screenHeight );
	winSize = ( screenWidth < screenHeight ) ? 
		screenWidth : screenHeight;
	winWidth = screenWidth / 2; 
	winHeight = screenHeight / 2;
	auxInitPosition( screenWidth / 4, screenHeight / 4, 
		winWidth, winHeight );
	auxInitDisplayMode( AUX_RGBA | AUX_DEPTH | AUX_DOUBLE );
	auxInitWindow( "superfly    `a' `b' `c' `f' `i' `n' `r' `t' SPACE keys MIDDLE/LEFT/RIGHT -MOUSE" );

	/* all of the following input callback routines are 
	 * in flyutil.c  */
	auxReshapeFunc( resize );	
	auxKeyFunc( AUX_ESCAPE, auxQuit );
	
	auxKeyFunc( AUX_SPACE, setDrawStyle );
	auxKeyFunc( AUX_n, toggleDrawNormals );
	auxKeyFunc( AUX_a, setAutopilot );
	auxKeyFunc( AUX_b, toggleBlend );
	auxKeyFunc( AUX_c, toggleMovingClip );
	auxKeyFunc( AUX_f, toggleFog );
	auxKeyFunc( AUX_i, toggleRaster );
	auxKeyFunc( AUX_s, toggleCulling );
	auxKeyFunc( AUX_r, resetEye );
	auxKeyFunc( AUX_t, toggleTexture );
	auxKeyFunc( AUX_d, setDebug );
	auxKeyFunc( AUX_UP, incDensity );
	auxKeyFunc( AUX_DOWN, decDensity );
	auxKeyFunc( AUX_QUESTION, printHelp );
	auxKeyFunc( AUX_SLASH, printHelp );

	/* Set up functions using the mouse */
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, startMove );
	auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEUP, stopMove );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, startMoveBack );
	auxMouseFunc( AUX_MIDDLEBUTTON, AUX_MOUSEUP, stopMoveBack );
	auxMouseFunc( AUX_RIGHTBUTTON, AUX_MOUSEDOWN, startPicking );

	auxAnimation( AUX_ON ); 

	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClearDepth( 1.0 );

	glClear( GL_COLOR_BUFFER_BIT );
	auxSwapBuffers();
	fprintf( stdout,  "\nLoading textures\n");

	glEnable( GL_DEPTH_TEST );

	glMatrixMode( GL_PROJECTION );
	aspect = (GLfloat) winWidth / winHeight;
	gluPerspective( 45.0, aspect, 1.0, 50.0 );
	glMatrixMode( GL_MODELVIEW );

	initLighting();
	resetEye();
	printHelp();
	image = rgbReadImageFile("maxHeadroom.rgb", &imageWidth, &imageHeight);
	textureList = initTextureList( "flowers.rgb" );	
	/*
	 * setup environment (reflection) mapping.
	 */
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
}

/*  Initialize material property, light source, and lighting model, 
 */
GLvoid
initLighting( GLvoid )
{
	GLfloat mat_ambient[] = { 0.25, 0.05, 0.4, 1.0 };
	GLfloat mat_diffuse[] = { 0.5, 0.1, 0.8, 1.0 };

	/* mat_specular and mat_shininess are NOT default values */
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 10.0 };

	/* light_position is NOT default value */
	GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
	GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };

	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glShadeModel( GL_SMOOTH );
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

GLvoid
drawScene( GLvoid )
{
	GLdouble planeABCD[4] = {0.0, -1.0, 0.0, 1.5};
	static GLfloat clipSpin = 0.0;

	moveEye();
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();

		if ( clipMode == ATTACH_EYE )
		{
			glClipPlane (GL_CLIP_PLANE1, planeABCD);
		}

		/* accomplish a viewing transformation by moving
		 * the world relative to the eye
		 */
		glRotatef( pitch, 1.0, 0.0, 0.0 );
		glRotatef( yaw, 0.0, 1.0, 0.0 );
		glTranslatef(-ex,-ey,-ez);

		if ( clipMode == FIXED_IN_SCENE )
		{
			glClipPlane (GL_CLIP_PLANE1, planeABCD);
		}

		else if ( clipMode == MOVING )
		{
			glPushMatrix();
				glTranslatef( 0.0, sinf( clipSpin ), 0.0 ); 
				glClipPlane (GL_CLIP_PLANE1, planeABCD);
				clipSpin = fmodf( clipSpin + 0.1, 2*M_PI );
			glPopMatrix();
		}

		if ( clipMode )
			glEnable (GL_CLIP_PLANE1);
		else
			glDisable (GL_CLIP_PLANE1);

		if ( rasterFlag )
		{
			static GLfloat rasterSpin = 0.0;
			glPushMatrix();
				glRotatef( -rasterSpin, 0, 1, 0 );
				glTranslatef(9.0, 1.0, 0.0);
				glRasterPos3f( 0.0, 0.0, 0.0 );
				glDrawPixels (imageWidth, imageHeight, GL_RGBA, 
					GL_UNSIGNED_BYTE, image);
			glPopMatrix();
			rasterSpin = fmodf((rasterSpin + velocity*10.0), 360.0);
		}
	
		glPushMatrix();
			if (picking)
				glLoadName(GRID);
			else
			    	computeGrid(); /* function in flyutil.c */	
			drawGrid(); /* function in flyutil.c */
		glPopMatrix();

		if ( cullingFlag && !picking && (autopilot || moving) )
			cullShapes();		
		drawShapes();

	glPopMatrix();

	glFlush();
	if ( !picking )
		auxSwapBuffers();
}

GLvoid
cullShapes( GLvoid )
{
	GLuint selectBuf[BUFSIZE];
	GLint hits;
	register int i;
	
	culling = GL_TRUE;
    
	glSelectBuffer (BUFSIZE, selectBuf);
	(GLvoid) glRenderMode( GL_SELECT );
		glInitNames();
		glPushName( -1 );
		/* set visibleFlags for all shapes */
		for (i = 0; i < (TORUS+ 1); i++)
			visibleFlag[i] = GL_TRUE;
		drawShapes();	
	hits = glRenderMode (GL_RENDER);
	cullingHits (hits, selectBuf);	
	culling = GL_FALSE;
	/* Just in case cullingMode was turned off while in cullShapes,
	 * if no longer in culling mode, set visibleFlags for all shapes
	 */
	if (!cullingFlag)
	{
		for (i = 0; i < (TORUS+ 1); i++)
			visibleFlag[i] = GL_TRUE;
	}
}

GLvoid 
cullingHits (GLint hits, GLuint buffer[] )
{
	register int i, j;
	int names;
	unsigned int *ptr;

	/* clear visibleFlags for all shapes */
	for (i = 0; i < (TORUS+ 1); i++)
		visibleFlag[i] = GL_FALSE;
	
	ptr = (unsigned int *) buffer;
	if (debugFlag)
		fprintf ( stdout, "culling hits = %d:  ", hits);
	for (i = 0; i < hits; i++) 
	{	/*  for each hit  */
		names = *ptr;
		ptr += 3; /* skip z1 and z2 */
		for (j = 0; j < names; j++) 
		{	/*  for each name */
			if (*ptr < (TORUS + 1))
			{
				if (debugFlag)
					fprintf (stdout, 
						"%s ", shapeNames[*ptr] );
				visibleFlag[*ptr] = GL_TRUE;
			}
			ptr++;
		}
	}
	if (debugFlag)
		fprintf ( stdout, "\n\n");
}



GLvoid
drawShapes( GLvoid )
{
	GLfloat shapesDiffuseAlpha[] = { 0.5, 0.1, 0.8, 0.3 };
	GLint currentShape;

	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, 
		shapesDiffuseAlpha);

	if ( blendFlag )
		disableBlending();

	if ( !culling && !calledRecursively )
	{
		if ( textureFlag )
		{
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glEnable(GL_TEXTURE_2D);
			glDisable(GL_LIGHTING);
			glColor4f(1, 1, 1, 0.3);
		}
	}

	currentShape = TETRA;

	drawMovedShape( -5.0, 1.0, 8.7, TETRA ); 
	drawMovedShape( -8.7, 1.0, 0.5, CUBE );
	drawMovedShape( 5.0, 1.0, 8.7, OCTA );
	drawMovedShape( -8.7, 1.0, -5.0, DODECA );
	drawMovedShape( -5.0,1.0,-8.7, ICOSA );
	drawMovedShape( 8.7,1.0,5.0, CONE );

	if ( blendFlag )
		enableBlending();
	drawMovedShape( 10.0,1.5,0.0, CYLINDER );
	if ( blendFlag )
		enableBlending();
	drawMovedShape( 7.0, 2.0, -7.0, SPHERE );
	if ( blendFlag )
		enableBlending();
	drawMovedShape( 0.0, 2.0, -10.0, TORUS );

	if ( blendFlag )
		disableBlending();

	if ( !culling && !calledRecursively )
	{
		if ( textureFlag )
		{
			glDisable(GL_TEXTURE_GEN_S);
			glDisable(GL_TEXTURE_GEN_T);
			glDisable(GL_TEXTURE_2D);
			glEnable(GL_LIGHTING);
		}
	}
}

GLvoid
drawMovedShape( GLfloat x, GLfloat y, GLfloat z, GLint currentShape )
{
	if ( !visibleFlag[currentShape] && !calledRecursively )
	{
		;  /* if not visible and not called recursively, don't draw */
	}
	else
	{
		glPushMatrix();
		glTranslatef(x, y, z);
		if (culling)
		{
			drawShape( currentShape, SIMPLE );
		}
		else if ((drawStyle == WIRE || pickedFlag[currentShape] ) && 
			(!picking)) 
		{
			if ( pickedFlag[currentShape] && !calledRecursively )
				drawSmallWorld();
			drawShape( currentShape, WIREFRAME );
		}
		else
		{
			drawShape( currentShape, SOLID );
		}
		glPopMatrix();
	}
}

GLvoid
drawShape( GLint currentShape, GLint drawMode )
{
	glLoadName(currentShape);
	switch( currentShape )
	{
		case TETRA:
			drawTetra( drawMode );
			break;
		case CUBE:
			drawCube( drawMode );
			break;
		case OCTA:
			drawOcta( drawMode );
			break;
		case DODECA:
			drawDodeca( drawMode );
			break;
		case ICOSA:
			drawIcosa( drawMode );
			break;
		case CONE:
			drawCone( drawMode );
			break;
		case CYLINDER:
			drawCylinder( drawMode );
			break;
		case SPHERE:
			drawSphere( drawMode );
			break;
		case TORUS:
			drawTorus( drawMode );
			break;
		default:
			break;
	}
}
		
GLvoid
drawTetra( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireTetrahedron( 1.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidTetrahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawCube( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireCube( 1.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidCube( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawOcta( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireOctahedron( 1.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidOctahedron( 1.0 );
			break;
		default:
			break;
	}
}					


GLvoid
drawDodeca( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireDodecahedron( 1.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidDodecahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawIcosa( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireIcosahedron( 1.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidIcosahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawCone( GLint drawMode )
{			
	glRotatef(-90.0, 1.0, 0.0, 0.0);
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireCone( 1.0, 2.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidCone( 1.0, 2.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawCylinder( GLint drawMode )
{			
	glRotatef(90.0, 1.0, 0.0, 0.0);
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireCylinder( 1.0, 2.0 );
			break;
		case SOLID:
		case SIMPLE:
			auxSolidCylinder( 1.0, 2.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawSphere( GLint drawMode )
{			
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireSphere( 1.0 );
			break;
		case SOLID:
			auxSolidSphere( 1.0 );
			break;
		case SIMPLE:
			auxSolidIcosahedron( 1.0 );
			break;
		default:
			break;
	}
}					

GLvoid
drawTorus( GLint drawMode )
{			
	static GLint spinTorus = 0;
	
	glRotatef( (GLfloat)spinTorus, 0.0, 1.0, 0.0 );
	switch( drawMode )
	{
		case WIREFRAME:
			auxWireTorus( 0.25, 0.75 );
			break;
		case SOLID:
			auxSolidTorus( 0.25, 0.75 );
			break;
		case SIMPLE:
			auxSolidBox ( 2.0, 2.0, 0.5 );
			break;
		default:
			break;
	}
	if ( !picking && !culling && !calledRecursively )
		spinTorus = ( spinTorus + (int)(20*velocity) ) % 360;
}

GLvoid 
drawSmallWorld( GLvoid )
{
	calledRecursively = GL_TRUE;
	/* need this because of the glScalef call */
	glEnable( GL_NORMALIZE );

	glPushMatrix();
		glScalef(0.075, 0.075, 0.075);
		drawShapes();
	glPopMatrix();
	glDisable( GL_NORMALIZE );
	calledRecursively = GL_FALSE;
}

GLvoid
toggleRaster( GLvoid )
{
	rasterFlag = !rasterFlag;
}

GLvoid
toggleTexture( GLvoid )
{
	textureFlag = !textureFlag;
	fprintf( stdout, "textureFlag = %d\n", textureFlag );
	if (textureFlag)
		glCallList(textureList); 
}

GLvoid
toggleBlend( GLvoid )
{
	blendFlag = !blendFlag;
	fprintf( stdout, "blendFlag = %d\n", blendFlag );
	if ( !blendFlag )
		disableBlending();
}

GLvoid
toggleFog( GLvoid )
{
	GLfloat fogColor[] = { 0.5, 0.5, 0.5, 1.0 };
	fogFlag = !fogFlag;
	if ( fogFlag )
	{
		glClearColor( fogColor[0], fogColor[1], fogColor[2], 
			fogColor[3] );
		glFogi (GL_FOG_MODE, GL_EXP);
		glFogfv (GL_FOG_COLOR, fogColor);
		glEnable( GL_FOG );		
		glFogf (GL_FOG_DENSITY, fogDensity);
	}
	else
	{
		glClearColor( 0.0, 0.0, 0.0, 1.0 );
		glDisable( GL_FOG );		
	}
}


GLvoid
decDensity( GLvoid )
{
	fogDensity -= 0.02;
	if (fogDensity < 0.0)
	fogDensity = 0.0;

	glFogf (GL_FOG_DENSITY, fogDensity);
	printf ("Fog density = %4.2f\n", fogDensity);
}

GLvoid
incDensity( GLvoid )
{
	fogDensity += 0.02;
	if (fogDensity > 1.0)
	fogDensity = 1.0;

	glFogf (GL_FOG_DENSITY, fogDensity);
	printf ("Fog density = %4.2f\n", fogDensity);
}

GLvoid
toggleMovingClip( GLvoid )
{
	/* OFF, ATTACH_EYE, FIXED_IN_SCENE, MOVING */
	
	clipMode++;
	if ( clipMode > MOVING )
		clipMode = OFF;
	
	switch( clipMode )
	{
		case OFF:
			fprintf( stdout, "clipMode = OFF\n" );
			break;
		case ATTACH_EYE:
			fprintf( stdout, "clipMode = ATTACH_EYE\n" );
			break;
		case FIXED_IN_SCENE:
			fprintf( stdout, "clipMode = FIXED_IN_SCENE\n" );
			break;
		case MOVING:
			fprintf( stdout, "clipMode = MOVING\n" );
			break;
		default:
			break;
	}
}


GLvoid
toggleCulling( GLvoid )
{
	register int i;
	
	cullingFlag = !cullingFlag;
	fprintf( stdout, "cullingFlag = %d\n", cullingFlag ); 
	/* set visibleFlags for all shapes */
	for (i = 0; i < (TORUS+ 1); i++)
		visibleFlag[i] = GL_TRUE;
}

GLvoid  
startPicking( AUX_EVENTREC *event )
{
	GLuint selectBuf[BUFSIZE];
	GLint hits;
	GLint viewport[4];
	GLint mouseX, mouseY;
	GLfloat aspect;

	mouseX = event->data[AUX_MOUSEX];
	mouseY = event->data[AUX_MOUSEY];
	
	picking = GL_TRUE;
	glGetIntegerv (GL_VIEWPORT, viewport);
	aspect = (GLfloat) viewport[2] / (GLfloat) viewport[3];
    
	glSelectBuffer (BUFSIZE, selectBuf);
	(GLvoid) glRenderMode( GL_SELECT );
	glInitNames();
	glPushName( -1 );
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
		glLoadIdentity();
		gluPickMatrix( (GLdouble) mouseX, 
			(GLdouble) (viewport[3] - mouseY), 
			5, 5,  /* x, y pixel tolerance for pick */
			viewport );
		gluPerspective( 45.0, aspect, 1.0, 50.0 );
		glMatrixMode( GL_MODELVIEW );
		drawScene();

		glFlush();
		hits = glRenderMode (GL_RENDER);
		printHits (hits, selectBuf);	
		picking = GL_FALSE;
		glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );
}

GLvoid 
printHits (GLint hits, GLuint buffer[] )
{
	unsigned int i, j, names, ii, jj;
	unsigned int *ptr;
	static GLdouble depthNormalizeFactor;
	static GLboolean init = GL_FALSE;
	GLint depthBits;  
	GLdouble depthNormalized;
	
	if (!init)
	{
		init = GL_TRUE;
		glGetIntegerv( GL_DEPTH_BITS, &depthBits );
		depthNormalizeFactor = pow(2, depthBits) - 1;
		fprintf( stdout, 
		"\ndepthBits=%d, depthNormalizeFactor=%f\n", 
		depthBits, depthNormalizeFactor); 
	}
	
	printf ("hits = %d\n", hits);
	ptr = (unsigned int *) buffer;
	for (i = 0; i < hits; i++) 
	{	/*  for each hit  */
		names = *ptr; ptr++;
		depthNormalized = (GLdouble)(*ptr)/depthNormalizeFactor; ptr++;
		printf ("\tz1 is %f;", depthNormalized);
		depthNormalized = (GLdouble)(*ptr)/depthNormalizeFactor; ptr++;
		printf ("\tz2 is %f;\n", depthNormalized);
		printf ("\tHit name:");
		for (j = 0; j < names; j++) 
		{	/*  for each name */
			if (*ptr < (TORUS + 1))
			{
				fprintf (stdout, 
					"  %s", shapeNames[*ptr] );
				pickedFlag[*ptr] = !pickedFlag[*ptr];
			}
			ptr++;
		}
	printf ("\n\n");
	}
}

GLvoid enableBlending( GLvoid )
{
	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	/* disable writing to the depth buffer so that transparent
	 * shapes don't block other shapes 
	 */
	glDepthMask( 0 );
}

GLvoid disableBlending( GLvoid )
{
	glDisable( GL_BLEND );
	/* reset blending function to default */
	glBlendFunc( GL_ONE, GL_ZERO );
	/* enable writing to the depth buffer */
	glDepthMask( 1 );
}

GLint initTextureList( char *imageFileName )
{
	GLuint imageList;
	int imageWidth, imageHeight ;
	GLuint *textureImage;

	textureImage = rgbReadImageFile(imageFileName, 
		&imageWidth, &imageHeight);
	imageList = glGenLists(1);
		glNewList(imageList, GL_COMPILE);
	/* scale texture image, make mipmaps and load texture
	 *	gluBuild2DMipmaps( target, components, width, height,
	 *		format,  type, imageArray ) 
	 */
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, imageWidth, imageHeight,
			 GL_RGBA, GL_UNSIGNED_BYTE, textureImage);
	glEndList();
	return( imageList );
}

GLvoid resetEye( GLvoid )
{
	register int i;

	/* set visibleFlags for all shapes */
	for (i = 0; i < (TORUS+ 1); i++)
		visibleFlag[i] = GL_TRUE;

	initializeEye();
}
