/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/* mipmap.c
 * This program reads in a texture from a .rgb file, uses the utility
 * routine gluBulid2dMipmaps() to scale and load mipmaps for a single 
 * texture image, then applies the texture to a rectangular polygon
 * using explicit texture coordinates.
 */
 
#include <aux.h>
#include <math.h>
#include <stdio.h>
/* rgbImageFile.h should be found in the ../../include directory */
#include "rgbImageFile.h"

/*  Function Prototypes  */

GLvoid  initialize( int, char ** );
GLvoid  drawScene( GLvoid );
GLvoid 	perspectiveViewing( GLfloat );
GLvoid  resize( GLsizei, GLsizei );
GLvoid  initTexture( unsigned int *, GLsizei, GLsizei );


main ( int argc, char *argv[])
{
	initialize( argc,  argv );

	auxMainLoop( drawScene );
}

GLvoid
initialize( int argc, char *argv[] )
{
	char		*imageFileName = "fish.rgba";
	unsigned int 	*image;
	GLsizei		imageWidth, imageHeight;
	GLint		screenWidth, screenHeight;
	GLsizei		windowWidth, windowHeight;
	GLfloat		aspect;
	
	if (argc < 2) {
		fprintf (stderr, "usage: %s <imageFileName>\n", argv[0] );
	} else
		imageFileName = argv[1];

	fprintf(stdout, "using image %s\n", imageFileName );

	image = rgbReadImageFile(imageFileName, &imageWidth,
			 &imageHeight);

	auxGetScreenSize( &screenWidth, &screenHeight );
	windowWidth = screenWidth / 2;
	windowHeight = screenHeight / 2;
	aspect = (GLfloat)windowWidth / (GLfloat)windowHeight;

	auxInitPosition( screenWidth / 4, screenHeight / 4, windowWidth, 
		windowHeight );
	auxInitDisplayMode( AUX_RGBA | AUX_DOUBLE );
	auxInitWindow( argv[0] );

	auxAnimation( AUX_ON );

	auxReshapeFunc( resize );

	auxKeyFunc( AUX_ESCAPE, auxQuit );

	glClearColor( 0, 0, 0, 1 );

	perspectiveViewing( aspect );

	initTexture( image, imageWidth, imageHeight );
}


GLvoid 
initTexture( unsigned int *image, 
	GLsizei imageWidth, GLsizei imageHeight )
{
	/* scale texture image, make mipmaps and load texture
	 *	gluBuild2DMipmaps( target, components, width, height,
	 *		format,  type, imageArray ) 
	 */
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, imageWidth, imageHeight,
			 GL_RGBA, GL_UNSIGNED_BYTE, image);
	
	glEnable(GL_TEXTURE_2D);
}


GLvoid
resize( GLsizei width, GLsizei height )
{
	GLfloat     aspect;
	glViewport( 0, 0, width, height );

	aspect = (GLfloat) (width) / height;
	perspectiveViewing( aspect );
}


GLvoid
perspectiveViewing( GLfloat aspect )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0, aspect, 1.0, 50.0 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -12.0 ); 
}
GLvoid
drawScene(void)
{
	static GLfloat swim;

	static float v0[3] = {-1.5,-1.0,0.0};
	static float v1[3] = {1.5,-1.0,0.0};
	static float v2[3] = {1.5,1.0,0.0};
	static float v3[3] = {-1.5,1.0,0.0};

	static float t0[3] = {0.0, 0.0};
	static float t1[3] = {1.0, 0.0};
	static float t2[3] = {1.0, 1.0};
	static float t3[3] = {0.0, 1.0};

	glClear(GL_COLOR_BUFFER_BIT);

	glColor4f( 1.0, 1.0, 1.0, 1.0 );
	glPushMatrix ();
		glTranslatef( -6.0 + swim, 0.2*sinf(swim * 3.0), -swim );
	
		glNormal3f(0,0,-1);
		glBegin (GL_QUADS);
			glTexCoord2fv(t0);
			glVertex3fv(v0);
			glTexCoord2fv(t1);
			glVertex3fv(v1);
			glTexCoord2fv(t2);
			glVertex3fv(v2);
			glTexCoord2fv(t3);
			glVertex3fv(v3);
		glEnd();

	glPopMatrix ();

	auxSwapBuffers();
	
	swim = fmodf( swim + 0.1, 35.0 );
}
